// Prevent console window in addition to Slint window in Windows release builds when, e.g., starting the app via file manager. Ignored on other platforms.
#![cfg_attr(not(debug_assertions), windows_subsystem = "windows")]

mod gui;
use slint::Model;
use std::{cell::RefCell, path::Path, rc::Rc};

slint::include_modules!();

struct GUIElements {
    vram_file_list:  Rc<slint::VecModel<slint::StandardListViewItem>>,
    vram_image_list: Rc<slint::VecModel<VRAMImage>>
}

impl GUIElements {
    pub fn new(main_window: &MainWindow) -> GUIElements {
        let vram_file_list:Vec<slint::StandardListViewItem> = main_window.get_main_tab_vram_file_list().iter().collect();
        let vram_file_list                                  = Rc::new(slint::VecModel::from(vram_file_list));

        let vram_image_list = Rc::new(slint::VecModel::from(Vec::<VRAMImage>::new()));

        main_window.set_main_tab_vram_file_list(vram_file_list.clone().into());
        main_window.set_main_tab_vram_images(vram_image_list.clone().into());
        GUIElements{vram_file_list, vram_image_list}
    }

    pub fn add_new_vram_file(&mut self, file: &str) {
        let vram_image = VRAMImage{
            img: slint::Image::load_from_path(Path::new("/home/jaby/Desktop/PSX_Dev/jabyengine/examples/PoolBox/assets/IMG_6921.png")).unwrap(),
            x:   0,
            y:   0
        };

        self.vram_file_list.push(slint::StandardListViewItem::from(file));
        self.vram_image_list.push(vram_image);
    }

    pub fn remove_vram_file(&mut self, idx: usize) {
        self.vram_file_list.remove(idx);
        self.vram_image_list.remove(idx);
    }

    pub fn move_vram_image(&mut self, idx: usize, dx: i32, dy: i32) {
        if let Some(mut vram_info) = self.vram_image_list.row_data(idx) {
            vram_info.x += dx;
            vram_info.y += dy;

            if vram_info.x < 0 {
                vram_info.x = 0;
            }

            if vram_info.y < 0 {
                vram_info.y = 0;
            }

            let (vram_img_width, vram_img_height) = (vram_info.img.size().width as i32, vram_info.img.size().height as i32);
            if (vram_info.x + vram_img_width) > 1024 {
                vram_info.x = 1024 - vram_img_width;
            }

            if (vram_info.y + vram_img_width) > 512 {
                vram_info.y = 512 - vram_img_height;
            }

            self.vram_image_list.set_row_data(idx, vram_info);
        }
    }

    pub fn create_bg() -> Result<slint::Image, slint::PlatformError> {
        Ok(slint::Image::from_rgba8_premultiplied(gui::create_vram_bg(320, 256).ok_or(slint::PlatformError::Other("Failed creating VRAM background image".to_string()))?))
    }
}

fn main() -> Result<(), slint::PlatformError> {
    let main_window  = MainWindow::new()?;
    let gui_elements = Rc::new(RefCell::new(GUIElements::new(&main_window)));

    main_window.set_main_tab_vram_bg(GUIElements::create_bg()?);

    let gui_elements_cloned = gui_elements.clone();
    main_window.on_main_tab_add_file_clicked(move || {
        gui_elements_cloned.borrow_mut().add_new_vram_file("Planschi");
    });
    let gui_elements_cloned = gui_elements.clone();
    main_window.on_main_tab_remove_file_clicked(move |idx: i32| {
        if idx >= 0 {
            gui_elements_cloned.borrow_mut().remove_vram_file(idx as usize);
        }
    });
    let gui_elements_cloned = gui_elements.clone();
    main_window.on_move_vram_image(move |idx: i32, dx: i32, dy: i32| {
        gui_elements_cloned.borrow_mut().move_vram_image(idx as usize, dx, dy);
    });

    main_window.run()
}